<div class="panel panel-default">
    <div class="panel-heading">Agent Status</div>
    <div class="panel-body">
        <table class="table">
            <thead>
                <tr>
                    <th></th>
                    <th>Is Running</th>
                    <th>Queue Size</th>
                    <th>Total Processed</th>
                    <th>Last Error</th>
                </tr>
            </thead>

            <tbody>
                <!-- Disp Compiler -->
                <tr>
                    <th>Display Compiler</th>
                    <td>{{ item.displayCompilerQueueStatus }}</td>
                    <td>{{ item.displayCompilerQueueSize }}</td>
                    <td>{{ item.displayCompilerProcessedTotal }}</td>
                    <td>{{ item.displayCompilerLastError }}</td>
                </tr>

                <!-- Grid Compiler -->
                <tr>
                    <th>Grid Compiler</th>
                    <td>{{ item.gridCompilerQueueStatus }}</td>
                    <td>{{ item.gridCompilerQueueSize }}</td>
                    <td>{{ item.gridCompilerProcessedTotal }}</td>
                    <td>{{ item.gridCompilerLastError }}</td>
                </tr>

                <!-- Location Index Compiler -->
                <tr>
                    <th>Location Compiler</th>
                    <td>{{ item.locationIndexCompilerQueueStatus }}</td>
                    <td>{{ item.locationIndexCompilerQueueSize }}</td>
                    <td>{{ item.locationIndexCompilerProcessedTotal }}</td>
                    <td>{{ item.locationIndexCompilerLastError }}</td>
                </tr>

                <!-- Branch Index Compiler -->
                <tr>
                    <th>Branch Compiler</th>
                    <td>{{ item.branchIndexCompilerQueueStatus }}</td>
                    <td>{{ item.branchIndexCompilerQueueSize }}</td>
                    <td>{{ item.branchIndexCompilerProcessedTotal }}</td>
                    <td>{{ item.branchIndexCompilerLastError }}</td>
                </tr>
            </tbody>
        </table>
    </div>
</div>
