"""updated float precision in zoom levels

Peek Plugin Database Migration Script

Revision ID: a66e033b8227
Revises: 5307eb21c742
Create Date: 2022-07-04 10:21:09.213318

"""

# revision identifiers, used by Alembic.
revision = "a66e033b8227"
down_revision = "5307eb21c742"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "DispLevel",
        "minZoom",
        type_=sa.Float(precision="5"),
        nullable=True,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispLevel",
        "maxZoom",
        type_=sa.Float(precision="5"),
        nullable=True,
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("DispLevel", "minZoom", type_=sa.Float(), nullable=True)
    op.alter_column("DispLevel", "maxZoom", type_=sa.Float(), nullable=True)
    # ### end Alembic commands ###
