"""added textHStretch

Peek Plugin Database Migration Script

Revision ID: 47879bec4c5f
Revises: a66e033b8227
Create Date: 2022-07-04 11:39:51.785436

"""

# revision identifiers, used by Alembic.
from sqlalchemy import Float

revision = "47879bec4c5f"
down_revision = "a66e033b8227"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "DispCurvedText",
        sa.Column(
            "textHStretch", Float, doc="hs", nullable=False, server_default="1"
        ),
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column(
        "DispCurvedText",
        "textHStretch",
        schema="pl_diagram",
    )
    # ### end Alembic commands ###
